'use strict'
const _ = require('lodash'),
      Promise = require('bluebird'),
      logger = require('./logger')

/**
 * snowmix.commands - handles the manipulation of Snowmix commands (aka functions)
 * @example snowmix.commands.listCommand('Show').then(arrayOfLines => { ... })
 */
class SnowmixCommands {

    constructor(snowmix) {
        this.snowmix = snowmix
    }

    /**
     * List the lines of a command
     *
     * @param {String} commandName
     * @return {Promise} of an array where the first entry is always undefined.
     * If the command does not exist, returns undefined.
     * If the command has no contents, returns [undefined]
     * @example snowmix.commands.listCommand('Show').then(arrayOfLines => { ... })
     */
    listCommand(commandName, args) {
        let listCommand = `command list ${commandName}`
        return this.snowmix.sendCommand(listCommand, { tidy: true, expectMultiline: true })
        .then(response => {
            if (response.length === 1 && response[0].match(/Invalid parameters/))
                return undefined

            let lines = [undefined]
            response.forEach(line => {
                let matches = line.match(/^\s*(\d+):\s+(.+?)\s*$/)
                if (matches) {
                    let lineNumber = matches[1]
                    let actualLine = matches[2]
                    lines[lineNumber] = actualLine
                }
            })
            return lines
        })
    }

    /**
     * Create a command. Replaces any that already exist with this name.
     *
     * @param {String} commandName
     * @param {Array} Lines for command
     * @return {Promise}
     */
    createCommand(commandName, lines) {
        let linesToCreateCommand = [
            `command delete ${commandName}`,
            `command create ${commandName}`
            ]
            .concat(lines)
            .concat('command end')
        return this.snowmix.sendCommand(linesToCreateCommand)
    }

    /**
     * updates the 'Show' Snowmix command to contain the relevant overlay commands
     * @return {Promise}
     */
    updateShowCommand(contentType) {
        if (['vfeed', 'text'].indexOf(contentType) === -1)
            throw new Error('updateShowCommand() does not handle type ' + contentType)

        // The Show command must have 'vfeed overlay' first, then 'text overlay'
        // (otherwise the video will clobber the text)
        // When switching between vfeeds, add a line after the previous one
        // Then delete the previous one. This prevents a momentary black screen
        return this.findLineNumbersOfContentTypeInShowCommand(contentType)
        .then(lineNumbersOfThisContentType => {
            let maxLineNumber = lineNumbersOfThisContentType.length ? _.max(lineNumbersOfThisContentType) : 1
            let commandsToDoUpdate = []
            if (visibleIds.length) commandsToDoUpdate.push
                (`command addatline Show ${maxLineNumber+1} ${contentType} overlay ` + visibleIds.join(' '))
            if (lineNumbersOfThisContentType.length) {
                let commandsToDoDelete = this.createDeleteCommand('Show', lineNumbersOfThisContentType)
                commandsToDoUpdate = commandsToDoUpdate.concat(commandsToDoDelete)
            }
            console.log(123, contentType, 'produced', commandsToDoUpdate)
            return this.snowmix.sendCommand(commandsToDoUpdate, {tidy: true, expectResponse: false})
        })
    }

    /**
     * Given a line to match, and a command name, returns the matching line numbers and lines.
     * @private
     */
    findLineNumbersOfContentTypeInShowCommand(contentType) {
        // We're looking for lines in Show() that look like:
        //   text overlay 1 2 3
        // where 'text' can be 'vfeed' or 'image' and there can be 1 or more numbers
        return this.getLinesThatMatch(contentType + ' overlay', 'Show')
        .then(lineDetails => {
            return Object.keys(lineDetails) // just the line numbers
        })
    }

    /**
     * Given a line to match, and a command name, returns the matching line numbers and lines.
     * @private
     */
    getLinesThatMatch(regexp, commandName) {
        return this.listCommand(commandName)
        .then(arrayOfLines => {
            let matching = {}
            arrayOfLines.forEach((line, lineNumber) => {
                if (lineNumber === 0) return
                let match
                if (match = line.match(regexp)) matching[lineNumber] = match
            })

            return matching
        })
    }

    /**
     * Returns the command to delete 1+ lines of a command
     * @private
     */
    createDeleteCommand(commandName, lineNumbers) {
        return `command deleteline ${commandName} ${lineNumbers.sort(a-b)}`
    }

    /**
     * Returns commands to delete matching lines of a command
     * @private
     */
    // getCommandsToDeleteLinesThatMatch(regexp, commandName, offset) {
    //     if (!offset) offset = 0
    //     return this.getLinesThatMatch(regexp, commandName)
    //     .then(matching => {
    //         let lineNumbers = Object.keys(matching).map(_.toNumber)
    //
    //         // Delete in reverse order, otherwise they all shuffle down!
    //         let deleteCommands = lineNumbers.sort((a,b)=>b-a)
    //         .map(n => { return `command deleteline ${commandName} ${n+offset}` })
    //         console.log(11, regexp, commandName, offset, deleteCommands)
    //         return deleteCommands
    //     })
    // }

    /**
     * Resets the Show command to containing nothing apart from the essential 'loop'
     * @return {Promise}
     */
    resetShowCommand() {
        return this.snowmix.sendCommand([
            'command delete Show',
            'command create Show',
            'loop',
            'command end'
        ])
    }
}

module.exports = SnowmixCommands
